function buildClassInfo(constructor, ifaces, classDescription, contractID, classID, classOptions) {
    var classInfo = { };

    classInfo[constructor] = {
        getInterfaces: function (count) {
            count.value = ifaces.length;
            return ifaces;
        },

        getHelperForLanguage: function (language) {
            return null;
        },

        'classDescription': classDescription,
        'contractID': contractID,
        'classID':  Components.ID(classID),
        'implementationLanguage': Components.interfaces.nsIProgrammingLanguage.JAVASCRIPT,
        'constructor': constructor,
        'flags': 0
    };

    for (var option in classOptions) {
        var constr = classInfo[constructor];
        constr[option] = classOptions[option];
    }

    return classInfo;
}

function buildModule(classInfo, scripts, baseScripts) {
    var module = {
        'classInfo': classInfo,

        'scripts': scripts,
        'baseScripts': baseScripts,

        mUtilsLoaded: false,

        unregisterSelf: function (aComponentManager) {
            aComponentManager = aComponentManager
            .QueryInterface(Components.interfaces.nsIComponentRegistrar);
            for each (var component in this.classInfo) {
                aComponentManager.unregisterFactoryLocation(component.classID);
            }
        },

        registerSelf: function (aComponentManager,
            aFileSpec,
            aLocation,
            aType) {
            aComponentManager = aComponentManager
            .QueryInterface(Components.interfaces.nsIComponentRegistrar);

            for each (var component in this.classInfo) {
                aComponentManager.registerFactoryLocation(
                    component.classID,
                    component.classDescription,
                    component.contractID,
                    aFileSpec,
                    aLocation,
                    aType);
            }
        },

        getClassObject: function (aComponentManager, aCID, aIID) {
            return getCommonClassObject(this, aComponentManager, aCID, aIID);
        },

        canUnload: function(aComponentManager) {
            return true;
        },
		
        createInstance: function (aOuter, aIID) {
            if (aOuter != null)
                throw Components.results.NS_ERROR_NO_AGGREGATION;
			
			loadUtils(this);
			
			for each (var component in this.classInfo) {
				return (new (eval(component.constructor))()).QueryInterface(aIID);
			}
        },
        
        QueryInterface: function (aIID) {
            if (!aIID.equals(Components.interfaces.nsISupports) &&
                !aIID.equals(Components.interfaces.nsIFactory))
                throw Components.results.NS_ERROR_NO_INTERFACE;
            return this;
        }
    };

    return module;
}

function getCommonClassObject(module, aComponentManager, aCID, aIID) {
    if (!aIID.equals(Components.interfaces.nsIFactory)) {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    }

    loadUtils(module);

    for each (var component in module.classInfo) {
        if (aCID.equals(component.classID)) {
            return buildFactoryFor(module, eval(component.constructor));
        }
    }
    throw Components.results.NS_ERROR_NO_INTERFACE;
}

function buildFactoryFor(module, aConstructor) {
    var factory = {
        QueryInterface: function (aIID) {
            if (!aIID.equals(Components.interfaces.nsISupports) &&
                !aIID.equals(Components.interfaces.nsIFactory))
                throw Components.results.NS_ERROR_NO_INTERFACE;
            return this;
        },

        createInstance: function (aOuter, aIID) {
            if (aOuter != null)
                throw Components.results.NS_ERROR_NO_AGGREGATION;
            return (new aConstructor()).QueryInterface(aIID);
        }
    };
    return factory;
}

function loadUtils(module) {
    if (module.mUtilsLoaded) {
        return;
    }

    Components.utils.import("resource://calendar/modules/calUtils.jsm");
    Components.utils.import("resource://calendar/modules/calProviderUtils.jsm");
    Components.utils.import("resource://calendar/modules/calAuthUtils.jsm");

    cal.loadScripts(module.baseScripts, Components.utils.getGlobalForObject(module));//module.__parent__);

    // Now load gdata extension scripts. Note that unintuitively,
    // __LOCATION__.parent == . We expect to find the subscripts in ./../js
    let thisDir = __LOCATION__.parent.parent.clone();
    thisDir.append("js");
    cal.loadScripts(module.scripts,
    		Components.utils.getGlobalForObject(module),//module.__parent__,
        thisDir);

    module.mUtilsLoaded = true;
}

var isTB3 = true;

/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

var scripts = [
	"CacheManager.js"
];

var ifaces = [
	Components.interfaces.ICacheManager,
	Components.interfaces.nsISupport
];

var classOptions = {
    'flags': Components.interfaces.nsIClassInfo.SINGLETON
};

var classInfo = buildClassInfo("CacheManager", ifaces, "FusionworkX Cache Manager", "@fusionworkx.com/common/cachemanager;1", "{F2EB52AC-17E3-11DF-96CC-F28055D89593}", classOptions);

var cacheManagerModule = buildModule(classInfo, scripts, [ ]);

/** 
* XPCOMUtils.generateNSGetFactory was introduced in Mozilla 2 (Firefox 4, SeaMonkey 2.1). 
* XPCOMUtils.generateNSGetModule was introduced in Mozilla 1.9 (Firefox 3.0). 
*/ 
//function NSGetModule(aCompMgr, aFileSpec) {
function NSGetFactory(aCompMgr, aFileSpec) {
	return cacheManagerModule;
}

