/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

var contactListColumns = [ 1, 6 ];

var contactColumns = [
	1, 6, 500, 501, 502, 506, 507, 508, 509, 510, 511, 515, 517, 518, 519, 520,
	523, 525, 526, 527, 528, 542, 544, 548, 551, 555, 556, 557, 558, 569, 560,
	594,
];

var contact_field_mapping = {
	1: 'object_id',
	6: 'last_modified',
	500: 'display_name',
	501: 'first_name',
	502: 'last_name',
	506: 'street_home',
	507: 'postal_code_home',
	508: 'city_home',
	509: 'state_home',
	510: 'country_home',
	511: 'birthday',
	515: 'nickname',
	517: 'anniversary',
	518: 'note',
	519: 'department',
	520: 'position',
	523: 'street_business',
	525: 'postal_code_business',
	526: 'city_business',
	527: 'state_business',
	528: 'country_business',
	542: 'telephone_business1',
	544: 'fax_business',
	548: 'telephone_home1',
	551: 'cellular_telephone1',
	555: 'email1',
	556: 'email2',
	557: 'email3',
	558: 'url',
	560: 'telephone_pager',
	569: 'company',
	594: 'number_of_distribution_list'
};

function ContactRequestHelper(host, session, cookie, connection) {
	this.host = host;
	this.session = session;
	this.cookie = cookie;
	this.connection = connection;
}

ContactRequestHelper.prototype = {

	// ADD, MODIFIFY AND DELETE METHODS*/
	/*
	 * Add a contact on server
	 * param folder - The folder where the contact is added
	 * param contactData - The contact as javascript object
	 * param callbackSucces - The succes callback return the following parameters:
	 *		contactRequestHelper - This object
	 *		remoteId - The newly created contact Id from the server
	 *		remoteTimestamp - The timestamp if the contact on the server
	 * param callbackFailed - The failed callback return the following parameters:
	 *		contactRequestHelper - This object
	 *		error - The error parameter from the json response
	 */
	addContact: function(folder, contactData, callbackSuccess, callbackFailed, params) {
		FW_LOG("addContact");
		var url = this.host + "/ajax/contacts?action=new&session=" + this.session;
		contactData['folder_id'] = folder;
		saveContactHttpRequest(url, folder, 0, contactData, this, false, callbackSuccess, callbackFailed, params);
	},

	updateContact: function(folder, remoteId, contactData, callbackSuccess, callbackFailed, params) {
		FW_LOG("updateContact: remoteId=" + remoteId);
		var url = this.host + "/ajax/contacts?action=update&id=" + remoteId + "&folder=" + folder + "&timestamp=" + MAX_TIMESTAMP + "&session=" + this.session;
		saveContactHttpRequest(url, folder, remoteId, contactData, this, true, callbackSuccess, callbackFailed, params);
	},

	deleteContact: function(remoteId, folder, callbackSuccess, callbackFailed, params) {
		FW_LOG("deleteContact remoteId=" + remoteId + ", folder=" + folder);
		var url = this.host + "/ajax/contacts?action=delete&timestamp=" + MAX_TIMESTAMP + "&session=" + this.session;

		deleteContactHttpRequest(url, remoteId, folder, this, callbackSuccess, callbackFailed, params);
	}
}

// #### HTTP REQUEST METHODS ####
function saveContactHttpRequest (url, folder, remoteId, contactData, contactRequestHelper, isUpdate, callbackSuccess, callbackFailed, params) {
	var request = getRequest();
	request.open("PUT", url, false);
	request.setRequestHeader("cookie", contactRequestHelper.cookie);
	request.overrideMimeType('text/javascript');
    
       
	request.onreadystatechange = function() {
		saveContactHttpReady(folder, remoteId, request, contactRequestHelper, isUpdate, callbackSuccess, callbackFailed, params);
	};
	

	request.send(FXJSON.toString(contactData, []));
    //request.send('{"color_label":"0","title":"Herr","last_name":"Mister","first_name":"Tester","display_name":"Mister, Tester","telephone_business1":"12345857","cellular_telephone1":"mail@tester.de","folder_id":"60","private_flag":false}');
    //FW_LOG();
	//saveContactHttpReady(folder, remoteId, request, contactRequestHelper, isUpdate, callbackSuccess, callbackFailed, params);
}

function saveContactHttpReady(folder, remoteId, request, contactRequestHelper, isUpdate, callbackSuccess, callbackFailed, params) {

    //FW_LOG("response" + request.getAllResponseHeaders());
    if(request.readyState == 4 && request.status == 200) {
		FW_LOG("saveContactReady for remote id " + remoteId + " response: " + request.responseText);
		var response = Function("return " + request.responseText)();
		var error = response['error'];

		if (error) {
			FW_ERROR("server error detected: " + error);
			callbackFailed(contactRequestHelper, error, params);
			return;
		} else {
			var data = response['data'];
			var timestamp = response['timestamp'];

			if (!isUpdate) {
				if (data['id']) {
					remoteId = data['id'];
                    FW_LOG("read contact remote_id from response: " + remoteId);
				} else {
					FW_ERROR("id not found in json response object: " + data.toSource() + ", isUpdate:" + isUpdate);
					callbackFailed(contactRequestHelper, "no id found in json response", params);
					return;
				}
			}

            FW_LOG("saveContactHttpReady: call success callback for id: " + remoteId);

			callbackSuccess(contactRequestHelper, remoteId, timestamp, params);
		}
	}
}

function deleteContactHttpRequest(url, remoteId, folder, contactRequestHelper, callbackSuccess, callbackFailed, params) {
	FW_LOG("deleteContactHttpRequest");

	var data = {
		'id': remoteId,
		'folder': folder
	};

	var request = getRequest();

	request.open("PUT", url, false);
	request.setRequestHeader("cookie", contactRequestHelper.cookie);
	request.overrideMimeType('text/javascript');
	request.send(FXJSON.toString(data, []));

	deleteContactHttpReady(request, remoteId, folder, contactRequestHelper, callbackSuccess, callbackFailed, params);
}

function deleteContactHttpReady(request, remoteId, folder, contactRequestHelper, callbackSuccess, callbackFailed, params) {
	if(request.readyState == 4 && request.status == 200) {
		FW_LOG("deleteContactReady remoteId=" + remoteId + ", folder=" + folder);
		var response = Function("return " + request.responseText)();
		var error = response['error'];

		if (error) {
			var errorCode = response['code'];

			if (errorCode == 'CON-0239' || errorCode == 'APP-0001') {
				FW_LOG("locally deleted object was already deleted on server");
				callbackSuccess(contactRequestHelper, params);
			} else if (errorCode == 'CON-0246') {
				FW_LOG("no permission to delete objects: " + response.toSource());
				callbackFailed(contactRequestHelper, error, params);
			} else {
				FW_ERROR("server error detected: " + response['error']);
				callbackFailed(contactRequestHelper, error, params);
			}
		} else {
			callbackSuccess(contactRequestHelper, params);
		}
	}
}


