/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

var ImipSyncOptions = {
    'system_sync_enabled': false,
    'calendar_sync_enabled': true,
    'addressbook_sync_enabled': false
};

function searchItemInLocalStorage(first, mailItem, itemFoundCallback, noItemCallback) {
    FW_LOG('searchItemInLocalStorage: first=' + first);

    FW_LOG("search for item : " + mailItem.title + " in local storage");
    var startAsLong = dateTime2Date(mailItem.startDate).getTime();
    var endAsLong = dateTime2Date(mailItem.endDate).getTime();

    var connection = getOXCalendarDBConnection();

    var calendar;

    var localId = mailItem.id;
    var folderId = existsAppointmentId(connection, localId);

    FW_LOG("check folder id after existsAppointmentId: folderId=" + folderId);

    if (folderId != -1) {
        calendar = searchItemCalendarByLocalId(connection, localId);
        itemFoundCallback(mailItem, calendar);
    }

    var appointments = searchAppointmentsByTitle(connection, mailItem.title, startAsLong, endAsLong);

    FW_LOG("found " + appointments.length + " appointments after search appointments by title");

    if (appointments.length == 0) {
        if (first) {
            FW_LOG("item not found in local storage, try to sync the storage again");

            noItemCallback(first, mailItem);
        } else {
            FW_LOG("item still not found, even after synching the local storage again");

            // call noItemCallback to continue working chain
            noItemCallback(first, mailItem);
        }
    } else {
        if (appointments.length > 1) {
            FW_LOG("more than one possible item found, use first one for accept or decline");
        } else {
            FW_LOG("one item found, using for accept or decline");
        }
        
        var appointment = appointments[0];
        localId = appointment['local_id'];
        calendar = searchItemCalendarByLocalId(connection, localId);
        if (calendar) {
            FW_LOG("sync object for appointment found, try to get item with id " + localId + " from calendar");

            var onGetItemListener = {
                onOperationComplete: function (aCalendar, aStatus, aOperationType, aId, aDetail) {
                    // call old setupbar method to continue the working chain 
                    // check if this.theItem exists to call the item found callback or
                    // call the noItemCallback
                    if (this.theItem) {
                        itemFoundCallback(this.theItem, calendar);
                    } else {
                        noItemCallback(first, mailItem);
                    }
                },

                onGetResult: function (aCalendar, aStatus, aItemType, aDetail, aCount, aItems) {
                    if (aStatus == Components.results.NS_OK && aCount == 1 && aItems.length == 1) {
                        FW_LOG("search item im local store successfully: aItems[0].id=" + aItems[0].id);

                        this.theItem = aItems[0];
                    } else {
                        FW_LOG("get item doesn't return the requested item");
                    }
                }
            };

            FW_LOG("read item from local calendar: calendar=" + calendar + ", localId=" + localId);

            // request local item from calendar with localId
            calendar.getItem(localId, onGetItemListener);
        } else {
            FW_LOG("no sync object found for appointment");
        }
    }
}

function searchItemCalendarByLocalId(connection, localId) {
    FW_LOG("searchItemCalendarByLocalId: localId=" + localId);
    var calendars = getCalendarManager().getCalendars({});

    for each (var calendar in calendars) {
        if (calendar.type == "oxcalendar") {
            var folder = getFolderFromCalendar(connection, calendar);
            var syncObject = getCalendarSyncObjectByLocalId(connection, folder, localId);
            if (syncObject) {
                return calendar;
            }
        }
    }

    return undefined;
}

function setOXAttendeeStatus(status, mailItem, calendar) {
    if (mailItem.id) {
        FW_LOG("setOXAttendeeStatus: " + status + ", item has no id");
    } else {
        FW_LOG("setOXAttendeeStatus: " + status + ", mailItem.id=" + mailItem.id);
    }
    
    var newItem = mailItem.clone();

    newItem.status = status;

    var listener = {
        onOperationComplete: function (aCalendar, aStatus, aOperationType, aId, aDetail) {
            var i18nConfirmationMessage;

            if (aStatus == Components.results.NS_OK) {
                i18nConfirmationMessage = getI18nString('oxcalendar-provider.confirmation.successful');
            } else {
                i18nConfirmationMessage = getI18nString('oxcalendar-provider.confirmation.failed');
            }

            var imipBar = document.getElementById("imip-bar");
            imipBar.setAttribute("label", i18nConfirmationMessage);

            hideElement("imip-button1");
            hideElement("imip-button2");
            hideElement("imip-button3");
        }
    };

    calendar.modifyItem(newItem, mailItem, listener);
}