/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

function SessionHandler(host, username, password, connection) {
    this.host = host;
    this.username = username;
    this.password = password;
    this.connection = connection;
}

SessionHandler.prototype = {

    setSession: function(session, cookie) {
        this.session = session;
        this.cookie = cookie;
    },

    /**
     * OX6 validate session method
     * @param callback   The callback of this method has the parameter 'success' if the login was successfully
     * @param params	 The parameter given to the callback method
     * @param async      Executes the http requests in async mode
     */
    validateSession: function(callback, params, async) {
        async = true;
        FW_LOG("ox6 validate session");
        FW_LOG("ox6 validate session: host=" + this.host + ", user=" + this.username);

        sessionHandlerConfigHttpRequest(this.host, this.session, this.cookie, this, callback, params, async);
    },

    login: function(callback, params, async) {
        async=true;
        FW_LOG("sessionHandler.login async=" + async);
        var usernameEncoded = encodeURIComponent(this.username);
        var passwordEncoded = encodeURIComponent(this.password);

        var url = this.host + "/ajax/login?action=login";
        var httpParams = "name="+usernameEncoded+"&password="+passwordEncoded;
                 
        sessionHandlerLoginHttpRequest(url, httpParams, this, callback, params, async);
    },

    logout: function(async) {
        async=true;
        FW_LOG("sessionHandler.logout async=" + async);
        sessionHandlerLogoutHttpRequest(this.host, this.session, this.cookie, 0, async);
    }
}

// #### HTTP REQUEST METHODS ####
function sessionHandlerLoginHttpRequest(url, httpParams, sessionHandler, callback, params, async) {
    async=true; //always set to true to prevent a thunderbird freeze
    var request = getRequest();
    
    FW_LOG("no existing session found, try to create a new one");
    request.open("POST", url , async); 
    request.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
    request.setRequestHeader("Content-length", httpParams.length);
    request.setRequestHeader("Connection", "close");
    request.overrideMimeType('text/javascript');

    if (async) {
        request.onreadystatechange = function() {
            
            sessionHandlerLoginHttpReady(request, sessionHandler, callback, params);
        };
    }
	FW_LOG(httpParams);
    request.send(httpParams);

    if (!async) {
        sessionHandlerLoginHttpReady(request, sessionHandler, callback, params);
    }
}

function sessionHandlerLoginHttpReady(request, sessionHandler, callback, params) {

    if(request.readyState == 4 && request.status == 200) {
        FW_LOG("http login event detected");

        var response = Function("return " + request.responseText)();
        if (response['session']) {
            FW_LOG("session found in login response");
            sessionHandler.session = response['session']

            
//            var newCookieList = [ ];
//
//            var cookieList = request.getResponseHeader("set-cookie").split("\n");
//            for (var a = 0; a < cookieList.length; a++) {
//                var cookieElements = cookieList[a].split(";");
//
//                for (var b = 0; b < cookieElements.length; b++) {
//                    var tmpCookie = cookieElements[b];
//                    if (tmpCookie.toUpperCase().indexOf("PATH=") == -1) {
//                        newCookieList[newCookieList.length] = tmpCookie;
//                    }
//                }
//            }
//
//            sessionHandler.cookie = newCookieList.join("; ");
//            FW_LOG("Cookie: " + sessionHandler.cookie);
            

            saveSession(sessionHandler.connection, sessionHandler.session, "");
            callback(sessionHandler, params, true);
        } else {
            FW_WARN("No session found, check response: " + request.responseText);
            callback(sessionHandler, params, false);
        }
    }
}


function sessionHandlerConfigHttpRequest(host, session, cookie, sessionHandler, callback, params, async) {
    async=true;
    var url = host + "/ajax/config/identifier?session=" + session;

    FW_LOG("oxConfigHttpRequest to url: " + url + " is async: " + async);

    var request = getRequest();
    request.open("GET", url, async);
    request.setRequestHeader("cookie", cookie);
    request.overrideMimeType('text/javascript');

    if (async) {
        request.onreadystatechange = function() {
            sessionHandlerConfigHttpReady(request, host, sessionHandler, callback, params, async);
        };
    }

    request.send(null);

    if (!async) {
        sessionHandlerConfigHttpReady(request, host, sessionHandler, callback, params, async);
    }
}

function sessionHandlerConfigHttpReady(request, host, sessionHandler, callback, params, async) {
    async=true;
    if (request.readyState == 4) {
        if (request.status == 200) {
            FW_LOG("sessionHandlerConfigHttpReady: " + request.responseText);
            var jsonResponse = Function("return " + request.responseText)();
            if (jsonResponse.code && (jsonResponse.code == 'SES-0202' || jsonResponse.code == 'SES-0203' ||  jsonResponse.code == 'SES-0206')) {
                FW_LOG("session not valid, response code: " + jsonResponse.code + " try to create a new session");

                // try to delete the current session from session pool
                var connection1 = getOXCalendarDBConnection();
                deleteSessionBySession(connection1, sessionHandler.session);

                sessionHandler.login(callback, params, async);
            } else {
                if (jsonResponse.error) {
                    FW_LOG("session not valid, response has errors: " + jsonResponse.error);

                    // try to delete the current session from session pool
                    var connection3 = getOXCalendarDBConnection();
                    deleteSessionBySession(connection3, sessionHandler.session);

                    callback(sessionHandler, params, false);
                } else {
                    FW_LOG("session is valid, keep session for next request");
                    callback(sessionHandler, params, true);
                }
            }
        }
    }
}

function sessionHandlerLogoutHttpRequest(host, session, cookie, id, async) {
    async=true;
    FW_LOG("ox6LogoutHttpRequest async=" + async);
    var url = host + "/ajax/login?session=" + session + "&action=logout";

    FW_LOG("logout url: " + url);
    var request = getRequest();
    request.open("GET", url, async);
    request.setRequestHeader("cookie", cookie);
    request.overrideMimeType('text/javascript');
    if (async) {
        request.onreadystatechange = function() {
            sessionHandlerLogoutHttpRequestReady(request, session, cookie, id);
        };
    }

    request.send(null);

    if (!async) {
        sessionHandlerLogoutHttpRequestReady(request, session, cookie, id);
    }
}

function sessionHandlerLogoutHttpRequestReady(request, session, cookie, id) {
    if (request.readyState == 4) {
        if (request.status == 200) {
            var jsonResponse = Function("return " + request.responseText)();
            if (!jsonResponse) {
                FW_LOG("session was successfully logged out");
                var connection = getOXCalendarDBConnection();
                if (id == 0) {
                    deleteSessionBySession(connection, session);
                } else {
                    deleteSession(connection, id);
                }
            } else {
                if (jsonResponse.error) {
                    FW_WARN("session was not logged out");
                }
            }
        }
    }
}

function logoutSessions(host, connection) {
    // try to logout all old sessions
    FW_LOG("try to logout old session");
    var oldSessions = getOldSessions(connection);
    FW_LOG(oldSessions.length + " old session found");
    for (var a = 0; a < oldSessions.length; a++) {
        var oldSession = oldSessions[a];
        var id = oldSession['id'];
        var logoutSession = oldSession['session'];

        FW_LOG("old sessions found, try to logout old session: " + logoutSession);

        var logoutCookie = oldSession['random'];
        sessionHandlerLogoutHttpRequest(host, logoutSession, logoutCookie, id);
    }
}
