/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

var logNetworkData = false;

var folderColumns = [ 1, 2, 5, 300, 301, 302, 308 ];

var folder_field_mapping = {
	1 : 'id',
	2 : 'created_by',
	5 : 'last_modified',
	300 : 'name',
	301 : 'module',
	302 : 'type',
	308 : 'default_folder'
};

var userColumns = [ 1, 500, 524, 555 ];

var user_field_mapping = {
	1 : 'id',
	500 : 'display_name',
	524 : 'internal_id',
	555 : 'email1'
};

function startSystemSync(syncManager) {
	FW_LOG("start system sync as background process");
	updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.reading-system-data'));

	var connection = getOXCalendarDBConnection();
	var host = getOXCalendarPref(connection, "host");
	var username = getOXCalendarPref(connection, "username");
	var password = getOXCalendarPref(connection, "password");

	var systemSync = new SystemSync(host, username, password, connection, syncManager);
	systemSync.login();
}

function SystemSync(host, username, password, connection, syncManager) {
	this.host = host;
	this.username = username;
	this.password = password;
	this.connection = connection;
	this.syncManager = syncManager;
}

SystemSync.prototype = {
	login : function() {
		FW_LOG("SystemSync.login host=" + this.host);

		// REMOVE var usernameEncoded = encodeURIComponent(this.username);
		// REMOVE var passwordEncoded = encodeURIComponent(this.password);

		var params = [ this ];
		this.sessionHandler = new SessionHandler(this.host, this.username, this.password, this.connection);
		var newestSession = getNewestSession(this.connection);
		this.sessionHandler.setSession(newestSession[0], newestSession[1]);
		this.sessionHandler.validateSession(systemSyncValidateSessionReady, params, false);

	},

	readInternalUserId : function() {
		FW_LOG("readInternalUserIdReady");

		var url = this.host + "/ajax/config/identifier?session=" + this.sessionHandler.session;

		ox6ReadInternalUserIdHttpRequest(url, this);
	},

	readInternalUserIdReady : function(request) {
		if (request.readyState == 4 && request.status == 200) {
			FW_LOG("readInternalUserIdReady event detected: " + request.responseText);

			var response = Function("return " + request.responseText)();
			var identifier = response['data'];

			saveOXCalendarPref(this.connection, "identifier", identifier);

			this.readContextId();
		}
	},

	readContextId : function() {
		FW_LOG("readContextId");

		var url = this.host + "/ajax/config/context_id?session=" + this.sessionHandler.session;

		ox6ReadContextIdHttpRequest(url, this);
	},

	readContextIdReady : function(request) {
		if (request.readyState == 4 && request.status == 200) {
			FW_LOG("readInternalUserIdReady event detected");

			var response = Function("return " + request.responseText)();
			var contextId = response['data'];

			saveOXCalendarPref(this.connection, "context_id", contextId);

			this.readUsers();
		}
	},

	readUsers : function() {
		FW_LOG("readUsers");

		updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.reading-system-users'));

		var participant_array = [];
		var lastsync = 0;

		var url = this.host + "/ajax/contacts?action=updates&session=" + this.sessionHandler.session + "&folder=6" + "&columns="
				+ encodeURIComponent(userColumns.join()) + "&timestamp=" + lastsync + "&ignore=deleted";

		// call http request not as async for TB3 because it is already threaded

		ox6ReadUsersHttpRequest(url, participant_array, this, false);

	},

	readUsersReady : function(request, participant_array) {
		if (request.readyState == 4 && request.status == 200) {
			FW_LOG("readUsersReady event detected");
			var response = Function("return " + request.responseText)();
			var userArray = response['data'];
			if (userArray) {
				for ( var a = 0; a < userArray.length; a++) {
					var user = userArray[a];
					var newUser = {};
					newUser['display_name'] = user[1];
					newUser['id'] = user[2];
					newUser['mail'] = user[3];
					newUser['type'] = 1;

					participant_array.push(newUser);
				}
			}

			FW_LOG("readUsersReady fetched " + participant_array.length + " participants from server");

			this.readGroups(participant_array);
		}
	},

	// GROUPS
	readGroups : function(participant_array) {
		updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.reading-system-groups'));

		var url = this.host + "/ajax/group?action=search&session=" + this.sessionHandler.session;

		// call http request not as async for TB3 because it is already threaded

		ox6ReadGroupsHttpRequest(url, participant_array, this, false);

	},

	readGroupsReady : function(request, participant_array) {
		if (request.readyState == 4 && request.status == 200) {
			var groupArray = Function("return " + request.responseText)()['data'];

			if (groupArray) {
				for ( var a = 0; a < groupArray.length; a++) {
					var group = groupArray[a];
					var id = group['id'];

					// ignore group with id 0
					if (id == 0) {
						continue;
					}

					var groupParticipant = {};
					var name = group['name'];

					groupParticipant['id'] = id;
					groupParticipant['display_name'] = group['display_name'];
					groupParticipant['name'] = name;
					groupParticipant['mail'] = name;
					groupParticipant['type'] = 2;

					participant_array.push(groupParticipant);
				}
			}

			FW_LOG("readGroupsReady fetched " + participant_array.length + " participants from server");

			this.readResources(participant_array);
		}
	},

	// RESOURCE
	readResources : function(participant_array) {
		updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.reading-system-resources'));

		var url = this.host + "/ajax/resource?action=search&session=" + this.sessionHandler.session;

		// call http request not as async for TB3 because it is already threaded

		ox6ReadResourcesHttpRequest(url, participant_array, this, false);

	},

	readResourcesReady : function(request, participant_array) {
		if (request.readyState == 4 && request.status == 200) {
			var resourceArray = Function("return " + request.responseText)()['data'];

			if (resourceArray) {
				for ( var a = 0; a < resourceArray.length; a++) {
					var resourceParticipant = {};
					var resource = resourceArray[a];
					var name = resource['name'];
					resourceParticipant['id'] = resource['id'];
					resourceParticipant['display_name'] = resource['display_name'];
					resourceParticipant['name'] = name;
					resourceParticipant['mail'] = name;
					resourceParticipant['type'] = 3;

					participant_array.push(resourceParticipant);
				}
			}

			FW_LOG("readResourcesReady fetched " + participant_array.length + " participants from server");

			saveParticipantData(participant_array, this.connection, this);
		}
	},

	readFolders : function() {
		FW_LOG("readFolders");

		updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.reading-system-folders'));

		var url = this.host + "/ajax/folders?action=updates" + "&ignore=deleted" + "&timestamp=0" + "&columns=" + encodeURIComponent(folderColumns.join())
				+ "&session=" + this.sessionHandler.session;

		// call http request not as async for TB3 because it is already threaded

		ox6ReadFoldersHttpRequest(url, this, false);

	},

	readFoldersReady : function(request) {
		if (request.readyState == 4 && request.status == 200) {
			FW_LOG("readFoldersReady event detected");
			var responseText = request.responseText;
			var response = Function("return " + responseText)();
			var data = response['data'];

			if (!data) {
				FW_WARN("folder response doesn't return data. Check json response: " + responseText);
				systemSyncReady(this.syncManager);

				return;
			}

			var folder_array = convert_arrays_to_objects(data, folderColumns, folder_field_mapping);
			var new_folder_array = [];

			for ( var a = 0; a < folder_array.length; a++) {
				var folder = folder_array[a];

				if (folder.default_folder && folder.type == 1) {
					FW_LOG("save private folder to preferences database: " + folder.module);
					saveOXCalendarPref(this.connection, folder.module + "_default_folder", folder.id);
					continue;
				}

				// TODO: remove this stuff if ox server SP5 is supported
				if (folder.module == "calendar" || folder.module == "tasks" || folder.module == "contacts") {
					var lastModifiedAsLong = folder.last_modified;
					var lastModified = new Date(lastModifiedAsLong);
					var lastModifiedOffset = lastModified.getTimezoneOffset() * 60 * 1000;
					lastModifiedAsLong -= lastModifiedOffset;
					folder.last_modified = lastModifiedAsLong;

					new_folder_array[new_folder_array.length] = folder;
				}
			}

			FW_LOG("readFoldersReady " + new_folder_array.length + " folders fetched from server");
			saveFolderData(new_folder_array, this.connection, this.syncManager);
		}
	}
};

// #### OX6 HTTP REQUEST ####
function ox6ReadInternalUserIdHttpRequest(url, systemSync, async) {
	var request = getRequest();
	request.open("GET", url, async);
	// request.setRequestHeader("cookie", systemSync.sessionHandler.cookie);
	request.overrideMimeType('text/javascript');

	if (async) {
		request.onreadystatechange = function() {
			systemSync.readInternalUserIdReady(request);
		};
	}

	request.send(null);

	if (!async) {
		systemSync.readInternalUserIdReady(request);
	}
}

function ox6ReadContextIdHttpRequest(url, systemSync, async) {
	var request = getRequest();
	request.open("GET", url, async);
	// request.setRequestHeader("cookie", systemSync.sessionHandler.cookie);
	request.overrideMimeType('text/javascript');

	if (async) {
		request.onreadystatechange = function() {
			systemSync.readContextIdReady(request);
		};
	}

	request.send(null);

	if (!async) {
		systemSync.readContextIdReady(request);
	}
}

function ox6ReadUsersHttpRequest(url, participantArray, systemSync, async) {
	var request = getRequest();
	request.open("GET", url, async);
	// request.setRequestHeader("cookie", systemSync.sessionHandler.cookie);
	request.overrideMimeType('text/javascript');

	// FIXME disabled sync-modus (since threading is disabled; see comment in
	// SyncManager.js)
	// if (async) {
	request.onreadystatechange = function() {
		systemSync.readUsersReady(request, participantArray);
	};
	// }

	request.send(null);

	// if (!async) {
	// systemSync.readUsersReady(request, participantArray);
	// }
}

function ox6ReadGroupsHttpRequest(url, participantArray, systemSync, async) {
	var request = getRequest();
	request.open("PUT", url, async);
	// request.setRequestHeader("cookie", systemSync.sessionHandler.cookie);
	request.overrideMimeType('text/javascript');

	if (async) {
		request.onreadystatechange = function() {
			systemSync.readGroupsReady(request, participantArray);
		};
	}

	request.send(FXJSON.toString({
		'pattern' : '*'
	}, []));

	if (!async) {
		systemSync.readGroupsReady(request, participantArray);
	}
}

function ox6ReadResourcesHttpRequest(url, participantArray, systemSync, async) {
	var request = getRequest();
	request.open("PUT", url, async);
	// request.setRequestHeader("cookie", systemSync.sessionHandler.cookie);
	request.overrideMimeType('text/javascript');

	if (async) {
		request.onreadystatechange = function() {
			systemSync.readResourcesReady(request, participantArray);
		};
	}

	request.send(FXJSON.toString({
		'pattern' : '*'
	}, []));

	if (!async) {
		systemSync.readResourcesReady(request, participantArray);
	}
}
function ox6ReadFoldersHttpRequest(url, systemSync, async) {
	var request = getRequest();
	request.open("GET", url, async);
	// request.setRequestHeader("cookie", systemSync.sessionHandler.cookie);
	request.overrideMimeType('text/javascript');

	if (async) {
		request.onreadystatechange = function() {
			systemSync.readFoldersReady(request);
		};
	}

	request.send(null);

	if (!async) {
		systemSync.readFoldersReady(request);
	}
}

// ##### HTTP CALLBACK METHOD ####
function systemSyncValidateSessionReady(sessionHandler, params, success) {
	if (success) {
		var systemSync = params[0];
		systemSync.readInternalUserId();
	} else {
		FW_WARN("cannot validate session, check credentials");
	}
}

function saveFolderData(folderArray, connection, syncManager) {
	FW_LOG("saveFolderData: syncManager=" + syncManager);

	var folderDetails = getAllFolderDetails(connection);
	var folderDict = {};

	for ( var a = 0; a < folderArray.length; a++) {
		var folder = folderArray[a];
		if (!folderDetails[folder.id]) {
			insertFolderDetails(connection, folder);
		} else {
			updateFolderDetails(connection, folder);
		}

		folderDict[folder.id] = true;
	}

	// check if some folders doesn't exists anymore
	for ( var id in folderDetails) {
		if (!folderDict[id]) {
			deleteFolderMapping(connection, id);
		}
	}

	systemSyncReady(syncManager);
}

function saveParticipantData(participantArray, connection, systemSync) {
	var newConnection = getOXCalendarDBConnection();

	var timer = new Date();

	var participantDetails = getAllParticipantDetails(newConnection);
	var participantDict = {};

	FW_LOG("reading participants took: " + (new Date().getTime() - timer.getTime()) + " ms");

	timer = new Date();

	FW_LOG("saveParticipantData: participant_array.length: " + participantArray.length);

	var insertCounter = 0;
	var updateCounter = 0;
	var deleteCounter = 0;

	// use transaction handler
	newConnection.beginTransaction();
	for ( var a = 0; a < participantArray.length; a++) {
		var participant = participantArray[a];
		// FW_LOG("read participant id=" + participant.id + ", display_name=" +
		// participant.display_name + ", type=" + participant.type);

		var participantDetail = participantDetails[participant.id];
		if (!participantDetail) {
			insertParticipantMapping(connection, 0, participant.id, participant.display_name, participant.mail, participant.type);
			insertCounter++;
		} else {
			var update = checkParticipantAttribute(participant, participantDetail, "display_name");
			if (!update) {
				update = checkParticipantAttribute(participant, participantDetail, "mail");
			}

			// FW_LOG("participant update status: " + update);

			if (update) {
				updateParticipantMapping(connection, 0, participant.id, participant.display_name, participant.mail, participant.type);
				updateCounter++;
			}
		}
		participantDict[participant.id] = true;
	}

	// check if some participants doesn't exists anymore
	for ( var id in participantDetails) {
		var type = participantDetails[id].type;
		if (!participantDict[id]) {
			deleteParticipantMapping(newConnection, 0, id, type);
			deleteCounter++;
		}
	}

	FW_LOG("participant actions: insert=" + insertCounter + ", update=" + updateCounter + ", delete=" + deleteCounter);

	newConnection.commitTransaction();
	FW_LOG("saving participants took: " + (new Date().getTime() - timer.getTime()) + " ms");
	systemSync.readFolders();
}

function checkParticipantAttribute(participant1, participant2, attribute) {
	if (participant1[attribute] && participant2[attribute]) {
		if (participant1[attribute] != participant2[attribute]) {
			return true;
		}
	} else {
		return true;
	}
	return false;
}
