/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices 
 *    of the original copyright holder(s) for the original code used.
 *    
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 * 
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */


function AppointmentStorage() {
    this.items = [ ];
    this.participants = { };
    this.itemCache = { };
    this.syncObjects = [ ];
    this.syncObjectsByRemoteId = { };
}

AppointmentStorage.prototype = {
    addItem: function(data) {
        FW_LOG("add item to storage");
        this.items.push(data);
    },

    modifyItem: function(id, data) {
        FW_LOG("try to modify item from storage with id " + id);
        for (var a = 0; a < this.items.length; a++) {
            if (this.items[a]['object_id'] == id) {
                FW_LOG("object with given id found in local storage, try to update");
                this.items[a] = data;
                delete this.itemCache[id];
                break;
            }
        }
		
    //LOG("this.items: " + this.items.toSource());
    },

    deleteItem: function(id) {
        FW_LOG("delete item from storage");
        for (var a = 0; a < this.items.length; a++) {
            if (this.items[a]['object_id'] == id) {
                this.items.splice(a, 1);
                delete this.itemCache[id];
                break;
            }
        }
    },
    /**
	 * load an item by the given id
	 * return an array: The first value of the array is the position in the 
	 *                  internal storage. The second value is the item itself
	 */
    getItem: function(id) {
        for (var a = 0; a < this.items.length; a++) {
            if (this.items[a]['object_id'] == id) {
                return [ a, this.items[a] ];
            }
        }

        return undefined;
    },
	
    getItems: function() {
        return this.items;
    },
	
    countItems: function() {
        return this.items.length;
    },
	
    mergeItems: function(newItems) {
        var existingIds = { };

        for (var a = 0; a < newItems.length; a++) {
            var id = newItems[a]['object_id'];
            var itemTuple = this.getItem(id);
            if (itemTuple) {
                //LOG("item already exists in internal storage, just update item at pos");
                this.modifyItem(id, newItems[a]);
            } else {
                //LOG("add new item to internal storage");
                this.addItem(newItems[a]);
            }
			
            existingIds[id] = true;
        }
		
        for (var b = 0; b < this.items.length; b++) {
            var deleteId = this.items[b]['object_id'];
            if (!existingIds[deleteId]) {
                this.deleteItem(deleteId);
            }
        }
    },
	
    deleteItems: function() {
        this.items = [ ];
        this.itemCache = { };
    },
	
    setParticipants: function(participants) {
        this.participants = participants;
    },
	
    getParticipants: function() {
        return this.participants;
    },
	
    getItemFromCache: function(id) {
        return this.itemCache[id];
    },
	
    addItem2Cache: function(id, item) {
        this.itemCache[id] = item;
    },

    deleteItemFromCache: function(id) {
        delete this.itemCache[id];
    },
	
    getSyncObjectsByRemoteId: function() {
        return this.syncObjectsByRemoteId;
    },
	
    setSyncObjectsByRemoteId: function(syncObjectsByRemoteId) {
        this.syncObjectsByRemoteId = syncObjectsByRemoteId;
    }
};
