/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices 
 *    of the original copyright holder(s) for the original code used.
 *    
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 * 
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

/*
This file exists two times. The original files is in the /js/ directory and the copy is located
in the directory /chrome/content/
 */

var appointment_field_mapping = { 
    1: 'object_id',
    2: 'created_by',
    100: 'categories',
    101: 'private_flag',
    200: 'title',
    201: 'start_date',
    202: 'end_date',
    203: 'note',
    204: 'alarm',
    206: 'recurrence_id',
    209: 'recurrence_type',
    210: 'change_exceptions',
    211: 'delete_exceptions',
    212: 'days',
    213: 'day_in_month',
    214: 'month',
    215: 'interval',
    216: 'until',
    300: 'foldername',
    301: 'module',
    400: 'location',
    401: 'full_time',
    402: 'shown_as',
    220: 'participants',
    221: 'users',
    222: 'occurrences',
    223: 'uid'
};

function calOX6RemoteHandler(syncManager) {
    FW_LOG("create new instance of calOX6RemoteHandler for calendar");

	this.syncManager = syncManager;
    this.lastsync = 0;
    this.userLastSync = 0;
}

calOX6RemoteHandler.prototype = {

    /**
     * OX6 validate session method
     * @param host       The OX Host
     * @param username   The OX Username
     * @param password   The OX Password
     * @param callback   The callback of this method has the parameter 'success' if the login was successfully
     * @param params     The parameter of the callback method
     * @param async      Executes the http requests in async mode
     */
    validateSession: function(host, username, password, callback, params, async) {
        FW_LOG("ox6 validate session");
        FW_LOG("ox6 validate session: host=" + host + ", user=" + username);

        this.username = username;
        this.password = password;

        ox6ConfigHttpRequest(host, this.session, this.cookie, this, callback, params, async);
    },

    setSession: function(session, cookie) {
        this.session = session;
        this.cookie = cookie;
    },
	
    logout: function(host, parameterObj) {
        FW_LOG("logout call");
        ox6HandlerLogoutHttpRequest(host, this.session, this.cookie);
    	
        delete this.session;
        delete this.cookie;
    },
	
    insertAppointment: function ox6_insertAppointment(host, appointmentData) {
        appointmentData['ignore_conflicts'] = true;
        appointmentData['folder_id'] = this.calendarFolder;
		
        appointmentData = internalObject2ox6Object(appointmentData);
		
        LOG("ox6_insertAppointment: host=" + host + ", appointmentData=" +appointmentData.toSource());
        var url = host + "/ajax/calendar/?action=new&session=" + this.session;

        var request = getRequest();
        request.open("PUT", url, false);
        //request.setRequestHeader("cookie", this.cookie);
        request.overrideMimeType('text/javascript');
        request.send(FXJSON.toString(appointmentData, []));
    
        FW_LOG("add appointment response: " + request.responseText);
    
        var data = Function("return " + request.responseText)()['data'];
        if (data['error']) {
            FW_ERROR("cannot add appointment to server, check response: " + data.toSource());
        }
    
        return data['id'];
    },

    updateAppointment: function ox6_updateAppointment(host, appointmentData) {
        appointmentData['ignore_conflicts'] = true;

        var id = appointmentData['id'];
        appointmentData = internalObject2ox6Object(appointmentData);
		
        var inFolder = this.calendarFolder;

        var url = host + "/ajax/calendar/?action=update&session=" + this.session + "&id=" + id +
        "&folder=" + inFolder + "&timestamp=99999999999999";
        FW_LOG("ox6_updateAppointment URL: " + url);

        var request = getRequest();
        request.open("PUT", url, false);
        //request.setRequestHeader("cookie", this.cookie);
        request.overrideMimeType('text/javascript');
    	
        request.send(FXJSON.toString(appointmentData, []));
    
        FW_LOG("ox6_updateAppointment response: " + request.responseText);
		
        //var data = Function("return " + request.responseText)()['data'];
        return id;
    },
	
    deleteAppointment: function ox6_deleteAppointment(host, appointmentData) {
        FW_LOG("ox6_deleteAppointment");
        var id = appointmentData['id'];
        var folder = this.calendarFolder;
	
        var data = {
            'id': id,
            'folder': folder
        };

        if (appointmentData['recurrence_date_position']) {
            data['recurrence_date_position'] = appointmentData['recurrence_date_position'];
        }

        var url = host + "/ajax/calendar/?action=delete&timestamp=9999999999999&session=" + this.session;
        var request = getRequest();

        request.open("PUT", url, false);
        //request.setRequestHeader("cookie", this.cookie);
        request.overrideMimeType('text/javascript');
        request.send(FXJSON.toString(data, []));
    	
        var retval = Function("return " + request.responseText)()['data'];
    },
	
    readAppointments: function (host, callback, async) {
        FW_LOG("readAppointments for folder " + this.calendarFolder);
        var folder = this.calendarFolder;
        var rangeStartAsLong = getPrefRangeStartAsLong(this.syncManager.connectionDB);
        var rangeEndAsLong = getPrefRangeEndAsLong(this.syncManager.connectionDB);

        var url = host + "/ajax/calendar/?action=updates&session=" + this.session +
        "&folder=" + folder + "&columns=" + encodeURIComponent(appointmentColumns.join()) +
        "&timestamp=" + this.lastsync +"&ignore=deleted&recurrence_master=true";

        if (rangeStartAsLong && rangeEndAsLong) {
            url += "&start=" + rangeStartAsLong + "&end=" + rangeEndAsLong;
        }

		FW_LOG("read appointments url: " + url);

        ox6ReadAppointmentsHttpRequest(url, this.session, this.cookie, this.syncManager, folder, callback, async);
    }
}

function loginHttpReady (request, ox6Handler, callback, params) {
    if (request.readyState == 4 && request.status == 200) {
        FW_LOG("ox6_login ready event detected");

        var success = false;

        var response = Function("return " + request.responseText)();
        if (response['session']) {
            ox6Handler.session = response['session']

            var newCookieList = [ ];

            var cookieList = request.getResponseHeader("set-cookie").split("\n");
            for (var a = 0; a < cookieList.length; a++) {
                var cookieElements = cookieList[a].split(";");

                for (var b = 0; b < cookieElements.length; b++) {
                    var tmpCookie = cookieElements[b];
                    if (tmpCookie.toUpperCase().indexOf("PATH=") == -1) {
                        newCookieList[newCookieList.length] = tmpCookie;
                    }
                }
            }

            ox6Handler.cookie = newCookieList.join().replace(",", "; ");
            saveSession(ox6Handler.syncManager.connectionDB, ox6Handler.session, ox6Handler.cookie);
            FW_LOG("found cookie: " + ox6Handler.cookie);

            success = true;
        } else {
            FW_ERROR("No session found, check response: " + request.responseText);
            FW_LOG("try to notify 'messagebox_observer' observers");
            notifyObservers(null, "open_messagebox", [ 'oxcalendar-provider.error.invalid-credentials' ]);
        }

        if (callback) {
			// maybe we need a better workaround for this later
			if (ox6Handler.calendar) {
				callback(ox6Handler.calendar, success, params);
			} else {
				callback(ox6Handler.syncManager, success, params);
			}
        }
    }
}

function configHttpReady(request, ox6Handler, host, callback, params, async) {
    if (request.readyState == 4) {
        if (request.status == 200) {
            var jsonResponse = Function("return " + request.responseText)();
            if (jsonResponse.code && jsonResponse.code == 'SES-0202') {
                FW_LOG("session not valid, response code: " + jsonResponse.code + " try to create a new session");
                ox6LoginHttpRequest(host, ox6Handler.username, ox6Handler.password, ox6Handler, callback, params, async);
            } else if (jsonResponse.code && jsonResponse.code == 'SES-0203') {
                FW_LOG("session not valid, response code: " + jsonResponse.code + " try to create a new session");
                ox6LoginHttpRequest(host, ox6Handler.username, ox6Handler.password, ox6Handler, callback, params, async);
            } else {
                if (jsonResponse.error) {
                    FW_LOG("session not valid, response has errors: " + jsonResponse.error);
                    callback(ox6Handler.calendar, false, params);
                } else {
                    FW_LOG("session is valid, keep session for next request");
                    callback(ox6Handler.calendar, true, params);
                }
            }
        }
    }
}

function readAppointmentsHttpReady (request, syncManager, folderId, callback) {
    if (request.readyState == 4 && request.status == 200) {
        var response = Function("return " + request.responseText)();
        if (response['error']) {
            FW_ERROR("ox6 readAppointmentsReady server error detected: " + response['error']);
            
            // TODO: use here a callback failed variable
            readAppointmentsFailed(folderId, syncManager);
            return ;
        }

        var data = response['data'];
        var timestamp = response['timestamp'];

        this.lastsync = 0;

		FW_LOG("recieved " + data.length + " appointments from server");
        var appointment_array = convert_arrays_to_objects(data, appointmentColumns, appointment_field_mapping);
        for (var a = 0; a < appointment_array.length; a++) {
            var appointment = appointment_array[a];
            appointment = ox6Object2InternalObject(appointment);

            // SERVER BUG WORKAROUND:
            // changed recurring exceptions must not contain the attribute
            // 'changed_exceptions', remove this manually
            if (appointment['id'] != appointment['recurrence_id'] && appointment['changed_exceptions']) {
                FW_LOG("invalid value in server response, remove changed_exceptions attribute");
                delete appointment['change_exceptions'];
            }

            //FW_LOG("apointment data: " + data.toSource());

            appointment_array[a] = appointment;
        }

        FW_LOG("ox6 readAppointmentsReady");
        FW_LOG(appointment_array.length + " appointments converted from server response");
        
        callback(folderId, appointment_array, syncManager);
    }
}

// #### HELPER METHODS ####

function internalObject2ox6Object(event) {
    FW_LOG("internalObject2ox6Object");

    var fullTime = isFullTime(event['full_time']);
    if (!fullTime) {
        var startDateAsLong = event['start_date'];
        var endDateAsLong = event['end_date'];
	
        var startDate = new Date(startDateAsLong);
        var endDate = new Date(endDateAsLong);
		
        var startOffset = startDate.getTimezoneOffset()*60*1000;
        var endOffset = endDate.getTimezoneOffset()*60*1000;
	    
        startDateAsLong -= startOffset;
        endDateAsLong -= endOffset;
		
        event['start_date'] = startDateAsLong;
        event['end_date'] = endDateAsLong;
    }

    return event;
}

function ox6Object2InternalObject(event) {
    FW_LOG("ox6Object2InternalObject: " + event['object_id']);

    var fullTime = isFullTime(event['full_time']);
    if (!fullTime) {
        var startDateAsLong = parseInt(event['start_date']);
        var endDateAsLong = parseInt(event['end_date']);
	
        var startDate = new Date(startDateAsLong);
        var endDate = new Date(endDateAsLong);
		
        var startOffset = startDate.getTimezoneOffset()*60*1000;
        var endOffset = endDate.getTimezoneOffset()*60*1000;
	    
        startDateAsLong += startOffset;
        endDateAsLong += endOffset;
		
        event['start_date'] = startDateAsLong;
        event['end_date'] = endDateAsLong;
    }
	
    return event;
}

function logoutOldSessions(host, connection) {
    // try to logout all old sessions
    FW_LOG("try to logout old session");
    var oldSessions = getOldSessions(connection);
    FW_LOG(oldSessions.length + " old session found");
    for (var a = 0; a < oldSessions.length; a++) {
        var oldSession = oldSessions[a];
        var id = oldSession['id'];
        var logoutSession = oldSession['session'];

        FW_LOG("old sessions found, try to logout old session: " + logoutSession);

        var logoutCookie = oldSession['random'];
        ox6LogoutHttpRequest(host, logoutSession, logoutCookie, id);
    }
}

