/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices 
 *    of the original copyright holder(s) for the original code used.
 *    
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 * 
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

/*
 This file exists two times. The original files is in the /js/ directory and the copy is located
 in the directory /chrome/content/
 */

var logActionCounter = 0;

function convert_arrays_to_objects(data_arrays, cols, field_mapping) {
	var object_array = [];

	for ( var a = 0; a < data_arrays.length; a++) {
		object_array[a] = convert_array_to_object(data_arrays[a], cols, field_mapping);
	}

	return object_array;
}

function convert_array_to_object(data_array, cols, field_mapping) {
	var retval = {};

	for ( var a = 0; a < data_array.length; a++) {
		var field = field_mapping[cols[a]]
		retval[field] = data_array[a];
	}

	return retval;
}

function FW_LOG(message) {
	var loggingEnabled = cal.getPrefSafe("calendar.debug.log", false);

	if (loggingEnabled) {
		var aConsoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
		aConsoleService.logStringMessage(message);
	}
}

function FW_WARN(message) {
	WARN(message);
}

function FW_ERROR(message) {
	ERROR(message);
}

function cloneObject(obj) {
	var newObj = {};
	for ( var name in obj) {
		newObj[name] = obj[name];
	}

	return newObj;
}

function endsWith(search, pattern) {
	return (search.match(pattern + "$") == pattern);
}

function myParseBoolean(value) {
	if (value) {
		if (String(value) == 'true') {
			return true;
		}
	}
	return false;
}

// TODO: replace with calGetString from calUtils
function fxGetString(aBundleName, aStringName, aParams) {
	var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);

	try {
		var propName = "chrome://oxcalendar/locale/" + aBundleName + ".properties";
		var props = stringBundleService.createBundle(propName);

		if (aParams && aParams.length) {
			return props.formatStringFromName(aStringName, aParams, aParams.length);
		} else {
			return props.GetStringFromName(aStringName);
		}
	} catch (ex) {
		var s = "Failed to read '" + aStringName + "' from " + "'chrome://lightning/locale/" + aBundleName + ".properties'.";
		Components.utils.reportError(s + " Error: " + ex);
		return s;
	}
}

function getDisplayName4SharedFolder(connection, folder) {
	var displayName = folder.name;
	var participantDetail = getParticipantDetails(connection, 0, folder.created_by, 1);
	if (participantDetail) {
		displayName = folder.name + " ( " + participantDetail.display_name + " )";
	} else {
		FW_WARN("cannot resolve created id " + folder.created_by + " for shared folder: " + folder.name + ". Try to resolve folder owner");
		participantDetail = getParticipantDetails(connection, 0, folder.owner, 1);
		if (participantDetail) {
			displayName = folder.name + " ( " + participantDetail.display_name + " )";
		} else {
			FW_WARN("cannot resolve owner id " + folder.owner + " for shared folder: " + folder.name);
		}
	}

	return displayName;
}

function saveFolderPreferences4Calendar(folderElement, id) {
	FW_LOG("saveFolderPreferences4Calendar: id=" + id);
	var selectedItem = folderElement.selectedItem;
	if (selectedItem) {
		var folder = selectedItem.value;

		var connection = getOXCalendarDBConnection();
		var prefName = "oxcalendar." + id + ".folder";

		if (folder == 0) {
			FW_LOG("personal folder selected, don't save folder to preferences or delete existing entry");
			deleteOXCalendarPref(connection, prefName);
		} else {
			saveOXCalendarPref(connection, prefName, folder);
			FW_LOG("save " + prefName + " property: " + folder);
		}
	}
}

function hasOnlineConnection(host) {
	FW_LOG("checkOnlineConnection host: " + host);
	try {
		var loginUrl = host + "/ajax/login?connectionCheck=true";
		var request = getRequest();
		request.open("GET", loginUrl, false);
		request.overrideMimeType('text/plain');
		request.send(null);

		if (request.status >= 200) {
			return true;
		}

		var url = host + "/servlet/webdav.version";

		request = getRequest();
		request.open("GET", url, false);
		request.overrideMimeType('text/plain');
		request.send(null);

		if (request.status >= 200) {
			return true;
		}
	} catch (e) {
		FW_LOG("cannot connect to host: " + host + " start connector in offline mode");
	}
	return false;
}

function getI18nString(aStringName, aParams) {
	var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);

	try {
		var propName = "chrome://oxcalendar-provider/locale/oxcalendar.properties";
		var props = stringBundleService.createBundle(propName);

		if (aParams && aParams.length) {
			return props.formatStringFromName(aStringName, aParams, aParams.length);
		} else {
			return props.GetStringFromName(aStringName);
		}
	} catch (ex) {
		var s = "Failed to read '" + aStringName + "' from " + "'chrome://oxcalendar-provider/locale/oxcalendar.properties'.";
		Components.utils.reportError(s + " Error: " + ex);
		Components.utils.reportError('params: ' + aParams.toSource());
		return s;
	}
}

function myTrim(str) {
	if (str == null)
		return null;

	var startingIndex = 0;
	var endingIndex = str.length - 1;

	while (str.substring(startingIndex, startingIndex + 1) == ' ')
		startingIndex++;

	while (str.substring(endingIndex, endingIndex + 1) == ' ')
		endingIndex--;

	if (endingIndex < startingIndex)
		return '';

	return str.substring(startingIndex, endingIndex + 1);
}

function convertStrArray2IntArray(strArray) {
	var intArray = [];
	for ( var a = 0; a < strArray.length; a++) {
		intArray.push(Number(myTrim(strArray[a])));
	}

	return intArray;
}

/*
 * Helper class that access components class StatusBarHandler
 */
function updateStatusBarWithThread(text, style) {
	var iStatusBarHandler = Components.classes['@fusionworkx.com/common/statusbarhandler;1'].getService().wrappedJSObject;
	iStatusBarHandler.updateStatusBar(text, style);
}
