/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

function LocalAddressbookHandlerTB3(uri, refreshCardCache) {
    FW_LOG("create instance of LocalAddressBookHandlerTB3: uri=" + uri);

    this.uri = uri;
    this.addressbookDirectory = getAddressbookDirectory(uri);
    this.cardCache = { };

    if (refreshCardCache) {
        this.refreshCardCache();
    }
}

LocalAddressbookHandlerTB3.prototype = {
    refreshCardCache: function() {
        this.cardList = listTB3Cards(this.addressbookDirectory);

        var cards = this.cardList;
        for (var a = 0; a < cards.length; a++) {
            var uuid = this.getLocalIdFromCard(cards[a]);

            if (uuid) {
                this.cardCache[uuid] = cards[a];
            }
        }
    },

    searchCardByDisplayName: function(displayName) {
        var cards = this.cardList;
        for (var a = 0; a < cards.length; a++) {
            var cardDisplayName = cards[a].displayName;
            if (displayName == cardDisplayName) {
                return cards[a];
            }
        }
        return undefined;
    },

    loadCard: function(id) {
        if (this.cardCache[id]) {
            return this.cardCache[id];
        }
        return undefined;
    },

    listCards: function() {
        return this.cardList;
    },

    getLocalIdFromCard: function(card) {
        FW_LOG("try to get local ID from card");
        return card.getProperty(FUSIONSYNC_ID, "");
    },

    addCard: function(oxContact) {
        FW_LOG("addCard");

        var uuid = getUUID();

        var addressbookCard = Components.classes["@mozilla.org/addressbook/cardproperty;1"].createInstance(Components.interfaces.nsIAbCard);
        addressbookCard = oxContact2Card(oxContact, addressbookCard);

        this.saveCard(uuid, addressbookCard, false);
        this.cardCache[uuid] = addressbookCard;

        return uuid;
    },

    updateCard: function(id, oxContact) {
        var addressbookCard = this.loadCard(id);
        addressbookCard = oxContact2Card(oxContact, addressbookCard);
        if (addressbookCard) {
            this.saveCard(id, addressbookCard, true);
        } else {
            FW_LOG("updateCard: Cannot find contact with id " + id);
        }
    },

    saveCard: function(id, addressbookCard, isUpdate) {
        FW_LOG("saveCard: id=" + id + ", isUpdate=" + isUpdate);
        if (isUpdate) {
            FW_LOG("update card with id " + id);
            this.addressbookDirectory.modifyCard(addressbookCard);
            this.cardCache[id] = addressbookCard;
        } else {
            this.addUUID2Card(addressbookCard, id, false);
            this.addressbookDirectory.addCard(addressbookCard);
        }
    },

    addUUID2Card: function(card, id, autoSave) {
        FW_LOG("addUUID2Card: id=" + id + " , autoSave=" + autoSave);
        card.setProperty(FUSIONSYNC_ID , id);
        if (autoSave) {
            this.addressbookDirectory.modifyCard(card);
        }
    },

    deleteCard: function(id) {
        var cardsToDelete = Components.classes["@mozilla.org/array;1"].createInstance(Components.interfaces.nsIMutableArray);
        var addressbookCard = this.loadCard(id);

        if (addressbookCard) {
            cardsToDelete.appendElement(addressbookCard, false);
            this.addressbookDirectory.deleteCards(cardsToDelete);

            if (this.cardCache[id]) {
                delete this.cardCache[id];
            }

            FW_LOG("contact with id: " + id + " deleted");
        } else {
            FW_LOG("deleteContact: id not found");
        }
    },

    commit: function() {
    //this.addressbookDatabase.commit(2);
    },

    close: function() {
    //this.addressbookDatabase.close(true);
    }
}

function listTB3Cards(addressbookDirectory) {
    FW_LOG("listTB3Cards");

    var cards = [ ];

    // childCards is a simpleEnumeration
    var cardsEnum = addressbookDirectory.childCards;
    while (cardsEnum.hasMoreElements()) {
        var card = cardsEnum.getNext().QueryInterface(Components.interfaces.nsIAbCard);
        if (card) {
            FW_LOG("add card with id " + card.getProperty(FUSIONSYNC_ID, ""));


            if (!card.isMailList) {
                FW_LOG("card display_name: " + getCardDisplayName(card, true));
                cards.push(card);
            }
        }
    }

    return cards;
}
